/*
 * Copyright (C) 2020 ~ 2021 Uniontech Software Technology Co., Ltd.
 *
 * Author:     xushitong<xushitong@uniontech.com>
 *
 * Maintainer: max-lv<lvwujun@uniontech.com>
 *             lanxuesong<lanxuesong@uniontech.com>
 *             zhangsheng<zhangsheng@uniontech.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef DPROTOCOLMONITOR_H
#define DPROTOCOLMONITOR_H

#include "base/ddevicemonitor.h"

#include <QObject>

DFM_MOUNT_BEGIN_NS
class DProtocolMonitorPrivate;
class DProtocolMonitor : public DDeviceMonitor
{
    Q_OBJECT

public:
    DProtocolMonitor(QObject *parent = nullptr);
    ~DProtocolMonitor();
};
DFM_MOUNT_END_NS

#endif   // DBLOCKMONITOR_H
